@@ -31,6 +31,7 @@ import java.io.IOException;
 import java.math.BigInteger;
 import java.util.List;
 
+import org.apache.poi.common.usermodel.PictureType;
 import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
 import org.apache.poi.util.LocaleUtil;
 import org.apache.poi.util.Units;
@@ -446,6 +447,8 @@ class TestXWPFRun {
                 for (XWPFPicture pic : pictures) {
                     assertNotNull(pic.getPictureData());
                     assertEquals("DOZOR", pic.getDescription());
+                    assertEquals(5, pic.getPictureData().getPictureType());
+                    assertEquals(PictureType.JPEG, pic.getPictureData().getPictureTypeEnum());
                 }
 
                 count += pictures.size();
@@ -675,10 +678,10 @@ class TestXWPFRun {
     void testSetGetTextHighlightColor() throws IOException {
         try (XWPFDocument document = new XWPFDocument()) {
             final XWPFRun run = document.createParagraph().createRun();
-            assertSame(STHighlightColor.NONE, run.getTextHightlightColor());
+            assertSame(STHighlightColor.NONE, run.getTextHighlightColor());
             assertFalse(run.isHighlighted());
             run.setTextHighlightColor("darkGreen"); // See 17.18.40 ST_HighlightColor (Text Highlight Colors)
-            assertSame(STHighlightColor.DARK_GREEN, run.getTextHightlightColor());
+            assertSame(STHighlightColor.DARK_GREEN, run.getTextHighlightColor());
             assertTrue(run.isHighlighted());
             run.setTextHighlightColor("none");
             assertFalse(run.isHighlighted());
